# LaunchShipHandlers.py
# April 17, 2002
#
# by Evan Light aka sleight42, all rights reserved
#
# Permission to redistribute this code as part of any other packaging requires
# the explicit permission of the author in advance.
##############################################################################

import App
import loadspacehelper
import MissionLib
import QuickBattle.QuickBattle
import ftb.ShipManager
import ftb.GUIUtils

# import Custom.Carriers.Dauntless

# TODO: Have the Launchers/LauncherGroups own the buttons
# TODO: handle damage to shuttle bays by destroying ships contained within
# TODO: Re-enable collision detection after launch
# when damage == some integer increment X / Y where Y is the # of ships 
# contained in the bay

ET_LAUNCH_SHIP = App.Game_GetNextEventType()
ET_TOGGLE_LAUNCH_TYPE = App.Game_GetNextEventType()
ET_TEST_BUTTON = App.Game_GetNextEventType()

pToggleLaunchTypeButton = []
pLaunchButton = []

#print "ftb.LaunchShipHandlers"

#### EVENT HANDLERS ####

def ToggleLaunchType( pObject, pEvent):
    global pLaunchButton, pToggleLaunchTypeButton
    launcherIndex = pEvent.GetInt()
    pFTBCarrier = ftb.ShipManager.GetShip( MissionLib.GetPlayer())
    pFTBLauncher = pFTBCarrier.GetLaunchers()[launcherIndex]
    pFTBLauncher.NextLaunchType()
    RegenerateOurScienceMenu()

def MayLaunchShuttleAgain( pObject, pEvent):
    global pLaunchButton
    pSystem = App.ShipSubsystem_Cast( pEvent.GetDestination())
    pFTBLauncher = ftb.LauncherManager.GetLauncherBySystem( pSystem)
    pFTBLauncher.SetClearToLaunch( 1)
    pFTBCarrier = ftb.ShipManager.GetShip( MissionLib.GetPlayer())
    launcherIndex = -1 
    for index in range( len( pFTBCarrier.GetLaunchers())):
        #print pFTBCarrier.GetLaunchers()[index]
        if pFTBCarrier.GetLaunchers()[index].Equals( pFTBLauncher):
            launcherIndex = index
            #print "match"
            break
    if( launcherIndex == -1):
        return
    pLaunchButton[launcherIndex].SetEnabled()

def LaunchShip( pObject, pEvent):
    global pLaunchButton
    launcherIndex = pEvent.GetInt()
    pFTBCarrier = ftb.ShipManager.GetShip( MissionLib.GetPlayer())
    pFTBLauncher = pFTBCarrier.GetLaunchers()[launcherIndex]
    sShipName = pFTBLauncher.GetLaunchType()
    pFTBLauncher.LaunchShip( sShipName)
    if( pFTBLauncher.HasMoreLaunches( sShipName) == 0):
        pFTBLauncher.NextLaunchType()
    RegenerateOurScienceMenu()
    #pLaunchButton[launcherIndex].SetDisabled()

def MissionStart( pObject, pEvent):
    MissionLib.GetMission().AddPythonFuncHandlerForInstance( ET_LAUNCH_SHIP, __name__+".LaunchShip")
    MissionLib.GetMission().AddPythonFuncHandlerForInstance( ET_TOGGLE_LAUNCH_TYPE, __name__+".ToggleLaunchType")
    PreLoadAssets()
    RegenerateOurScienceMenu()

#### Handler Helpers #### 

def AddLaunchButtons( pMenu):
    #print "AddLaunchButtons"
    global pToggleLaunchTypeButton, pLaunchButton 
    pCarrier = ftb.ShipManager.GetShip( MissionLib.GetPlayer())
    pLaunchers = pCarrier.GetLaunchers()
    numTypes = len( pLaunchers)
    #print "num launchers:", numTypes

    pToggleLaunchTypeButton = []
    pLaunchButton = []
    for index in range( numTypes):
        #print "Adding for index", index
        buttonLabel = "Launch " + str(index + 1)
        pButton = ftb.GUIUtils.CreateIntButton( buttonLabel, ET_LAUNCH_SHIP, \
            MissionLib.GetMission(), index)
        pMenu.AddChild( pButton)
        pLowest = ftb.GUIUtils.FindLowestChild( pMenu)
        pButton.SetPosition( pLowest.GetLeft(), pLowest.GetBottom(), 0)
        pLaunchButton.append( pButton)

        launchType = pLaunchers[index].GetLaunchType()
        numLaunches =  pLaunchers[index].GetNumLaunches( launchType)
        buttonLabel = launchType + ": " + str( numLaunches)
        pButton = ftb.GUIUtils.CreateIntButton( buttonLabel, \
            ET_TOGGLE_LAUNCH_TYPE, MissionLib.GetMission(), index)
        pMenu.AddChild( pButton)
        pButton.AlignTo( pMenu.GetFirstChild(), App.TGUIObject.ALIGN_BL, App.TGUIObject.ALIGN_BR)
        pToggleLaunchTypeButton.append( pButton)
        if( numLaunches == 0):
            pButton.SetDisabled()

def RegenerateOurScienceMenu():
    #print "RegenerateOurScienceMenu:", MissionLib.GetPlayer().GetShipProperty().GetShipName()
    pScienceMenu = ftb.GUIUtils.GetScienceMenu()
    pScienceMenu.KillChildren()
    RebuildScienceMenuButtons()
    AddLaunchButtons( pScienceMenu)

    # KLUGE ALERT!  The below line seems to avoid the strange effect
    # where the last toggle button is created and added but doesn't appear
    # on screen.
    pScienceMenu.AddChild( None)
    
    # uhm need to restart Engineering :(
    import LoadQBautostart
    LoadQBautostart.ImportQBautostart(1,1)

def PreLoadAssets():
    "Cache up all of the launchable ships to avoid a framerate hit"
    pCarrier = ftb.ShipManager.GetShip( MissionLib.GetPlayer())
    pLaunchers = pCarrier.GetLaunchers()
    for launcher in pCarrier.GetLaunchers():
        for shipScript in launcher.GetComplement()._keyList.keys():
            #print "preloading %s %d times" % (shipScript, \
            #                             launcher.GetNumLaunches( shipScript))
            loadspacehelper.PreloadShip( shipScript, \
                                         launcher.GetNumLaunches( shipScript))

######################
# Partial implmentation of Bridge.ScienceMenuHandlers.CreateMenus()
######################
def RebuildScienceMenuButtons():
    # Get string database.
    pDatabase = App.g_kLocalizationManager.Load("data/TGL/Bridge Menus.tgl")

    # Get strings from database.
    kScience    = pDatabase.GetString("Science")
    kReport     = pDatabase.GetString("Report")
    kObject     = pDatabase.GetString("Scan Object")
    kTarget     = pDatabase.GetString("Scan Target")
    kArea       = pDatabase.GetString("Scan Area")
    kLaunch     = pDatabase.GetString("Launch Probe")

    pScienceMenu = ftb.GUIUtils.GetScienceMenu()

    from Bridge import BridgeMenus
    pCommunicate = BridgeMenus.CreateCommunicateButton("Science", pScienceMenu)
    pScienceMenu.AddChild(pCommunicate)
    
    from  Bridge import BridgeUtils
    pScienceMenu.AddChild(BridgeUtils.CreateBridgeMenuButton(kArea,     App.ET_SCAN,    App.CharacterClass.EST_SCAN_AREA,   pScienceMenu))

    pScanTarget = BridgeUtils.CreateBridgeMenuButton(kTarget,   App.ET_SCAN,    App.CharacterClass.EST_SCAN_OBJECT, pScienceMenu)
    pScienceMenu.AddChild(pScanTarget)
    pScanTarget.SetDisabled()

    pScanObjectMenu = App.STCharacterMenu_CreateW( kObject)
    pScienceMenu.AddChild(pScanObjectMenu)
    RebuildScanMenu( pScanObjectMenu)
    
    pLaunch = BridgeUtils.CreateBridgeMenuButton(kLaunch,   App.ET_LAUNCH_PROBE, 0, pScienceMenu)
    pScienceMenu.AddChild(pLaunch)

    if (App.g_kUtopiaModule.IsMultiplayer()):
        pCommunicate.SetDisabled()
        pLaunch.SetDisabled()

    #pScienceMenu.SetNoSkipParent()

def RebuildScanMenu( pScanMenu):
    # TODO: Add objects in range of the player ship to the scan menu
    pass

App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_MISSION_START, App.Game_GetCurrentGame(), __name__ + ".MissionStart")
#App.g_kEventManager.AddBroadcastPythonFuncHandler(MissionLib.ET_PLAYER_SHIP_CREATED, App.Game_GetCurrentGame(), __name__ + ".RegenerateOurScienceMenu")
#App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_TARGET_LIST_CLICKED, App.Game_GetCurrentGame(), __name__ + ".TargetListClicked")

def TargetListClicked( pObject, pEvent):
    casts = [App.TGEventHandlerObject_Cast, \
             App.ShipClass_Cast, \
             App.ShipSubsystem_Cast, \
             App.TGPane_Cast, \
             App.TGButton_Cast, \
             App.TGTextButton_Cast, \
             App.STMenu_Cast, \
             App.STButton_Cast, \
             App.TGIntEvent_Cast,\
             App.TGBoolEvent_Cast,\
             App.TGObjPtrEvent_Cast,\
             App.TGCharEvent_Cast,\
             App.TGShortEvent_Cast,\
             App.TGFloatEvent_Cast,\
             App.TGStringEvent_Cast]
    for cast in casts:
        obj = cast( pObject)
        if( obj != None):
            print "TargetListClicked pObject is a",obj
    for cast in casts:
        obj = cast( pEvent)
        if( obj != None):
            print "TargetListClicked pEvent is a",obj
            if( cast == App.TGIntEvent_Cast):
                print "TGIntEvent val", str( obj.GetInt())
    for cast in casts:
        obj = cast( pEvent.GetDestination())
        if( obj != None):
            print "TargetListClicked pEvent.GetDestination() is a",obj
    for cast in casts:
        obj = cast( pEvent.GetSource())
        if( obj != None):
            print "TargetListClicked pEvent.GetSource() is a",obj
