# Based on Banbury's QuickBattle Replacement Transporter

import App
import MissionLib
import Bridge.BridgeUtils
import Lib.Ambiguity
import QuickBattle.QuickBattle
import Custom.QuickBattleGame.QuickBattle

pBridge = App.g_kSetManager.GetSet('bridge')
g_pBrex = App.CharacterClass_GetObject(pBridge, 'Engineer')
pBrexMenu = Bridge.BridgeUtils.GetBridgeMenu('Engineer')
pGame = App.Game_GetCurrentGame()
pEpisode = pGame.GetCurrentEpisode()
pMission = pEpisode.GetCurrentMission()
pFriendlies = pMission.GetFriendlyGroup()

def Transport(pObject, pEvent):
	# First the Vars
	pGame		= App.Game_GetCurrentGame()
	pEpisode		= pGame.GetCurrentEpisode()
	pMission		= pEpisode.GetCurrentMission()
	pFriendlies		= pMission.GetFriendlyGroup()
	pPlayer		= MissionLib.GetPlayer()
	pTarget		= pPlayer.GetTarget()
	pSet 			= pPlayer.GetContainingSet()
	fTime			= 10 #Shields flicker Time
	g_pDatabase	= App.g_kLocalizationManager.Load("data/TGL/Bridge Crew General.tgl")
	pSetPlayerAction	= App.TGScriptAction_Create(__name__, "SetPlayerAction", MissionLib.GetShip(pTarget.GetName()))
	pFlickerShields	= App.TGScriptAction_Create("Actions.ShipScriptActions", "FlickerShields", 0, fTime)
	pBrexLine		= App.CharacterAction_Create(g_pBrex, App.CharacterAction.AT_SAY_LINE, "LoweringShields", None, 0, g_pDatabase)
	pTargetattr		= App.ShipClass_Cast(pTarget)
	iTargetIsNotFriendly	= None #an Indicator

	# Test if we have a Target
	if not pTarget:
		print("No Target - no Transport")
		return
	
	# Test Distance
	if (Distance(pTarget) > 300):
		print("Target is too far away: can't Transport")
		return
		
	# Test if our Target is friendly
	if (pFriendlies.IsNameInGroup(pTarget.GetName()) != 1):
		print("Target is not friendly...testing Shields")
		
		iTargetIsNotFriendly	= 1
		
		# try to find an offline Shield
		if (pTargetattr.GetShields() != None):
			if (pTargetattr.GetShields().GetCurShields(App.ShieldClass.FRONT_SHIELDS) < 200):
				print ("Front Shield is Offline...beaming")
			elif (pTargetattr.GetShields().GetCurShields(App.ShieldClass.REAR_SHIELDS) < 200):
				print ("After Shield is Offline...beaming")
			elif (pTargetattr.GetShields().GetCurShields(App.ShieldClass.TOP_SHIELDS) < 200):
				print ("Top Shield is Offline...beaming")
			elif (pTargetattr.GetShields().GetCurShields(App.ShieldClass.BOTTOM_SHIELDS) < 200):
				print ("Bottom Shield is Offline...beaming")
			elif (pTargetattr.GetShields().GetCurShields(App.ShieldClass.LEFT_SHIELDS) < 200):
				print ("Left Shield is Offline...beaming")
			elif (pTargetattr.GetShields().GetCurShields(App.ShieldClass.RIGHT_SHIELDS) < 200):
				print ("Right Shield is Offline...beaming")
			else:
				print ("No offline Shield...sorry")
				return
	
	# Go
	if (pTarget != None):
		MissionLib.RemoveCommandableShip(pTarget.GetName())
		MissionLib.AddCommandableShip(pPlayer.GetName())

		# Create Video Sequence
		pSequence = App.TGSequence_Create()
		
		# Flicker Player Shields when they are active
		if ( pPlayer.GetShields().IsOn() == 1 ):
			pSequence.AppendAction(pBrexLine) # Brex
			pSequence.AppendAction(pFlickerShields) # old Ship
		
		if (pFriendlies.IsNameInGroup(pTarget.GetName()) != 1):
			# Changing Target group to Friendly id it wasn't before
			pFriendlies.AddName(pTarget.GetName())
		
		# Some Video stuff
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.MissionScriptActions", "ChangeToBridge"), 1)
		pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "StartCutscene"))
		pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "SetTarget", pTarget.GetName()))
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "CutsceneCameraBegin", pSet.GetName()), 2)
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "ChangeRenderedSet", pSet.GetName()))
		
		# Beam
		pSequence.AppendAction(pSetPlayerAction)
				
		# Flicker Shields on new Ships if we have to
		if ( pTargetattr.GetShields().IsOn() == 1 and iTargetIsNotFriendly != 1):
			pSequence.AppendAction(pFlickerShields)
		
		# More Video stuff
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "LockedView", pSet.GetName(), pTarget.GetName(), 30, 30, 15))
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "CutsceneCameraEnd", pSet.GetName()), 6)	
		pSequence.AppendAction(App.TGScriptAction_Create("Actions.CameraScriptActions", "ChangeRenderedSet", "bridge"))
		pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "SetTarget", pPlayer.GetName()))
		pSequence.AppendAction(App.TGScriptAction_Create("MissionLib", "EndCutscene"))

		# Play
		pSequence.Play()
		
		# Perhaps Banbury tried here to reset the Players AI
		MissionLib.SetPlayerAI("Captain", None)
		pPlayer.ClearAI()

	if (pObject):
		pObject.CallNextHandler(pEvent)


# Beaming yeah!
def SetPlayerAction(pAction, pShip):
	pGame = App.Game_GetCurrentGame()
	
	# Play the sound
	pSound = App.TGSound_Create("sfx/Interface/new_game3.wav", "Tr_Sound", 0)
	pSound.SetSFX(0)
	pSound.SetInterface(1)
	App.g_kSoundManager.PlaySound("Tr_Sound")
	
	pGame.SetPlayer(pShip)
	return 0


# Get the Distance between the Player and pObject
def Distance(pObject):
	pPlayer = App.Game_GetCurrentGame().GetPlayer()
	vDifference = pObject.GetWorldLocation()
	vDifference.Subtract(pPlayer.GetWorldLocation())

	return vDifference.Length()


# The Button
if (pBrexMenu != None):
	event = Lib.Ambiguity.getEvent("ET_TRANSPORT")
	g_pBrex.AddPythonFuncHandlerForInstance(event, __name__ + ".Transport")
	pButton = QuickBattle.QuickBattle.CreateBridgeMenuButton(App.TGString('Transporter'), event, 0, g_pBrex)
	pBrexMenu.PrependChild(pButton)
